@@ -4,6 +4,7 @@
 Create Contours.
 """
 import warnings
+import math
 from abc import ABC, abstractmethod
 from multiprocessing import Pool, TimeoutError
 
@@ -22,9 +23,9 @@ class Contour(ABC):
 
     Attributes
     ----------
-    distribution : MultivariateDistribution,
+    distribution : MultivariateDistribution
         The distribution to calculate the contour of.
-    coordinates : list of lists of ndarrays,
+    coordinates : list of lists of ndarrays
         Contains the coordinates of points on the contour.
         If the distribution is unimodal the outer list contains only one element,
         else the outer list divides possible multiple contour parts.
@@ -33,8 +34,8 @@ class Contour(ABC):
 
     """
 
-    def __init__(self, mul_var_distribution, return_period=25, state_duration=3, timeout=1e6,
-                 *args, **kwargs):
+    def __init__(self, mul_var_distribution, return_period=25, state_duration=3,
+                 timeout=None, *args, **kwargs):
         """
 
         Parameters
@@ -43,7 +44,7 @@ class Contour(ABC):
             The distribution to be used to calculate the contour.
         Raises
         ------
-        TimeoutError,
+        TimeoutError
             If the calculation takes too long and the given value for timeout is exceeded.
         """
         self.distribution = mul_var_distribution
@@ -53,18 +54,22 @@ class Contour(ABC):
         self.return_period = return_period
         self.alpha = state_duration / (return_period * 365.25 * 24)
 
-        # Use multiprocessing to define a timeout
-        pool = Pool(processes=1)
-        res = pool.apply_async(self._setup, args, kwargs)
-        try:
-            computed = res.get(timeout=timeout)
-        except TimeoutError:
-            err_msg = "The calculation takes too long. " \
-                      "It takes longer than the given value for" \
-                      " a timeout, which is '{} seconds'.".format(timeout)
-            raise TimeoutError(err_msg)
-        # Save the results separated
-        self._save(computed)
+        if timeout:
+            # Use multiprocessing to define a timeout
+            pool = Pool(processes=1)
+            res = pool.apply_async(self._setup, args, kwargs)
+            try:
+                computed = res.get(timeout=timeout)
+            except TimeoutError:
+                err_msg = "The calculation takes too long. " \
+                          "It takes longer than the given value for" \
+                          " a timeout, which is '{} seconds'.".format(timeout)
+                raise TimeoutError(err_msg)
+            # Save the results separated
+            self._save(computed)
+        else:
+            computed = self._setup(*args, **kwargs)
+            self._save(computed)
 
     @abstractmethod
     def _setup(self, *args, **kwargs):
@@ -76,22 +81,29 @@ class Contour(ABC):
 
 
 class IFormContour(Contour):
-    def __init__(self, mul_var_distribution, return_period=25, state_duration=3, n_points=20,
-                 timeout=1e6):
+    def __init__(self, mul_var_distribution, return_period=25, state_duration=3,
+                 n_points=20, timeout=None):
         """
+        Contour based on the inverse first-order reliability method.
+
+        This method was proposed by Winterstein et al. (1993).
+
         Parameters
         ----------
-        mul_var_distribution : MultivariateDistribution,
+        mul_var_distribution : MultivariateDistribution
             The distribution to be used to calculate the contour.
         return_period : float, optional
             The years to consider for calculation. Defaults to 25.
         state_duration : float, optional
-            Time period for which a (environmental) state is measured, expressed in hours.
-            Defaults to 3.
+            Time period for which an environmental state is measured,
+            expressed in hours. Defaults to 3.
         n_points : int, optional
             Number of points on the contour. Defaults to 20.
         timeout : int, optional
-            The maximum time in seconds there the contour has to be computed. Defaults to 1e6.
+            The maximum time in seconds there the contour has to be computed.
+            This parameter also controls multiprocessing. If timeout is None
+            serial processing is performed, if it is not None multiprocessing
+            is used. Defaults to None.
         Raises
         ------
         TimeoutError,
@@ -112,34 +124,33 @@ class IFormContour(Contour):
         >>> loc = ConstantParam(0.8888)
         >>> scale = ConstantParam(2.776)
         >>> par1 = (shape, loc, scale)
-        >>> mu = FunctionParam(0.1000, 1.489, 0.1901, "f1")
-        >>> sigma = FunctionParam(0.0400, 0.1748, -0.2243, "f2")
+        >>> mu = FunctionParam('power3', 0.1000, 1.489, 0.1901)
+        >>> sigma = FunctionParam('exp3', 0.0400, 0.1748, -0.2243)
         >>> #Create distributions
         >>> dist1 = WeibullDistribution(*par1)
         >>> dist2 = LognormalDistribution(mu=mu, sigma=sigma)
         >>> distributions = [dist1, dist2]
         >>> dependencies = [dep1, dep2]
         >>> mul_dist = MultivariateDistribution(distributions, dependencies)
-        >>> test_contour_IForm = IFormContour(mul_dist, 50, 3, 400)
+        >>> test_contour_iform = IFormContour(mul_dist, 50, 3, 400)
 
         """
-        # TODO docuent state_duration
         # Calls _setup
         super().__init__(mul_var_distribution, return_period, state_duration, timeout, n_points)
 
     def _setup(self, n_points):
         """
-        Calculates coordinates using IForm method.
+        Calculates coordinates using IFORM.
 
         Parameters
         ----------
-        n_points : int,
+        n_points : int
             Number of points the shape contains.
-        return_period : float,
+        return_period : float
             The years to consider for calculation. Defaults to 25.
         Returns
         -------
-        tuple of objects,
+        tuple of objects
             The computed results.
         """
 
@@ -148,7 +159,7 @@ class IFormContour(Contour):
 
         distributions = self.distribution.distributions
 
-        beta = sts.norm.ppf(self.alpha)
+        beta = sts.norm.ppf(1 - self.alpha)
 
         # Create sphere
         if self.distribution.n_dim == 2:
@@ -181,20 +192,149 @@ class IFormContour(Contour):
 
         Parameters
         ----------
-        computed : tuple of objects,
+        computed : tuple of objects
             The computed results to be saved.
         """
         self.beta = computed[0]
         self.sphere_points = computed[1]
         self.coordinates = computed[2]
 
-class HighestDensityContour(Contour):
-    def __init__(self, mul_var_distribution, return_period=25, state_duration=3, limits=None,
-                 deltas=None, timeout=1e6):
+
+class ISormContour(Contour):
+    def __init__(self, mul_var_distribution, return_period=25, state_duration=3,
+                 n_points=20, timeout=None):
         """
+        Contour based on the inverse second-order reliability method.
+
+        This method was proposed by Chai and Leira (2018). The paper's DOI
+        is 10.1016/j.marstruc.2018.03.007 .
+
         Parameters
         ----------
-        mul_var_distribution : MultivariateDistribution,
+        mul_var_distribution : MultivariateDistribution
+            The distribution to be used to calculate the contour.
+        return_period : float, optional
+            The years to consider for calculation. Defaults to 25.
+        state_duration : float, optional
+            Time period for which an environmental state is measured,
+            expressed in hours. Defaults to 3.
+        n_points : int, optional
+            Number of points on the contour. Defaults to 20.
+        timeout : int, optional
+            The maximum time in seconds there the contour has to be computed.
+            This parameter also controls multiprocessing. If timeout is None
+            serial processing is performed, if it is not None multiprocessing
+            is used. Defaults to None.
+        Raises
+        ------
+        TimeoutError,
+            If the calculation takes too long and the given value for timeout is exceeded.
+
+        example
+        -------
+
+        >>> from viroconcom.distributions import (WeibullDistribution,\
+                                               LognormalDistribution,\
+                                               MultivariateDistribution)
+        >>> from viroconcom.params import ConstantParam, FunctionParam
+        >>> #Define dependency tuple
+        >>> dep1 = (None, None, None)
+        >>> dep2 = (0, None, 0)
+        >>> #Define parameters
+        >>> shape = ConstantParam(1.471)
+        >>> loc = ConstantParam(0.8888)
+        >>> scale = ConstantParam(2.776)
+        >>> par1 = (shape, loc, scale)
+        >>> mu = FunctionParam('power3', 0.1000, 1.489, 0.1901)
+        >>> sigma = FunctionParam('exp3', 0.0400, 0.1748, -0.2243)
+        >>> #Create distributions
+        >>> dist1 = WeibullDistribution(*par1)
+        >>> dist2 = LognormalDistribution(mu=mu, sigma=sigma)
+        >>> distributions = [dist1, dist2]
+        >>> dependencies = [dep1, dep2]
+        >>> mul_dist = MultivariateDistribution(distributions, dependencies)
+        >>> test_contour_isorm = ISormContour(mul_dist, 50, 3, 400)
+
+        """
+        # Calls _setup
+        super().__init__(mul_var_distribution, return_period, state_duration, timeout, n_points)
+
+    def _setup(self, n_points):
+        """
+        Calculates coordinates using ISORM.
+
+        Parameters
+        ----------
+        n_points : int
+            Number of points the shape contains.
+        return_period : float
+            The years to consider for calculation. Defaults to 25.
+        Returns
+        -------
+        tuple of objects
+            The computed results.
+        """
+
+        # Creates list with size that equals grade of dimensions used.
+        data = [None] * self.distribution.n_dim
+
+        distributions = self.distribution.distributions
+
+        # Use the ICDF of a chi-squared distribution with n dimensions. For
+        # reference see equation 20 in Chai and Leira (2018).
+        beta = math.sqrt(sts.chi2.ppf(1 - self.alpha, self.distribution.n_dim))
+
+        # Create sphere.
+        if self.distribution.n_dim == 2:
+            _phi = np.linspace(0, 2 * np.pi , num=n_points, endpoint=False)
+            _x = np.cos(_phi)
+            _y = np.sin(_phi)
+            _circle = np.stack((_x,_y)).T
+            sphere_points = beta * _circle
+
+        else:
+            sphere = NSphere(dim=self.distribution.n_dim, n_samples=n_points)
+            sphere_points = beta * sphere.unit_sphere_points
+
+        # Get probabilities for coordinates of shape.
+        norm_cdf_per_dimension = [sts.norm.cdf(sphere_points[:, dim])
+                               for dim in range(self.distribution.n_dim)]
+
+        # Inverse procedure. Get coordinates from probabilities.
+        for index, distribution in enumerate(distributions):
+            data[index] = distribution.i_cdf(norm_cdf_per_dimension[index], rv_values=data,
+                                             dependencies=self.distribution.dependencies[index])
+
+        coordinates = [data]
+
+        return (beta, sphere_points, coordinates)
+
+    def _save(self, computed):
+        """
+        Save the computed parameters.
+
+        Parameters
+        ----------
+        computed : tuple of objects
+            The computed results to be saved.
+        """
+        self.beta = computed[0]
+        self.sphere_points = computed[1]
+        self.coordinates = computed[2]
+
+
+class HighestDensityContour(Contour):
+    def __init__(self, mul_var_distribution, return_period=25, state_duration=3, limits=None,
+                 deltas=None, timeout=None):
+        """
+        Contour based on highest density contour method.
+
+        This method was proposed by Haselsteiner et al. (2017). The paper's
+        DOI is 10.1016/j.coastaleng.2017.03.002 .
+
+        Parameters
+        ----------
+        mul_var_distribution : MultivariateDistribution
             The distribution to be used to calculate the contour.
         return_period : float, optional
             The years to consider for calculation. Defaults to 25.
@@ -212,7 +352,10 @@ class HighestDensityContour(Contour):
             as there are dimensions in mul_var_dist.
             Defaults to 0.5.
         timeout : int, optional
-            The maximum time in seconds there the contour has to be computed. Defaults to 1e6.
+            The maximum time in seconds there the contour has to be computed.
+            This parameter also controls multiprocessing. If timeout is None
+            serial processing is performed, if it is not None multiprocessing
+            is used. Defaults to None.
         Raises
         ------
         TimeoutError,
@@ -221,7 +364,7 @@ class HighestDensityContour(Contour):
         example
         -------
 
-        Creating Contour example for 2-d HDC with Weibull and Lognormal
+        Creating contour example for 2-d HDC with Weibull and Lognormal
         distribution
 
         >>> from viroconcom.distributions import (WeibullDistribution,\
@@ -236,8 +379,8 @@ class HighestDensityContour(Contour):
         >>> loc = ConstantParam(0.8888)
         >>> scale = ConstantParam(2.776)
         >>> par1 = (shape, loc, scale)
-        >>> mu = FunctionParam(0.1000, 1.489, 0.1901, 'f1')
-        >>> sigma = FunctionParam(0.0400, 0.1748, -0.2243, 'f2')
+        >>> mu = FunctionParam('power3', 0.1000, 1.489, 0.1901)
+        >>> sigma = FunctionParam('exp3', 0.0400, 0.1748, -0.2243)
         >>> #Create distributions
         >>> dist1 = WeibullDistribution(*par1)
         >>> dist2 = LognormalDistribution(mu=mu, sigma=sigma)
@@ -264,11 +407,11 @@ class HighestDensityContour(Contour):
 
         Parameters
         ----------
-        limits : list of tuples,
+        limits : list of tuples
             One 2-Element tuple per dimension in mul_var_distribution,
             containing min and max limits for calculation. ((min, max)).
             Smaller value is always assumed minimum.
-        deltas : scalar or list of scalar,
+        deltas : scalar or list of scalar
             The grid stepsize used for calculation.
             If a single float is supplied it is used for all dimensions.
             If a list of float is supplied it has to be of the same length
@@ -281,7 +424,7 @@ class HighestDensityContour(Contour):
         if deltas is None:
             deltas = [0.5] * self.distribution.n_dim
         else:
-            # Check if deltas is a scalar
+            # Check if deltas is a scalar.
             try:
                 iter(deltas)
                 if len(deltas) != self.distribution.n_dim:
@@ -301,7 +444,7 @@ class HighestDensityContour(Contour):
                                  "but len(limits)={}, n_dim={}."
                                  "".format(len(limits), self.distribution.n_dim))
 
-        # Create sampling coordinate arrays
+        # Create sampling coordinate arrays.
         sample_coords = []
         for i, lim_tuple in enumerate(limits):
             try:
@@ -328,12 +471,12 @@ class HighestDensityContour(Contour):
             raise ValueError("Encountered nan in cell averaged probabilty joint pdf. "
                              "Possibly invalid distribution parameters?")
 
-        # Calculate probability per cell
+        # Calculate probability per cell.
         cell_prob = f
         for delta in deltas:
             cell_prob *= delta
 
-        # Calculate highest density region
+        # Calculate highest density region.
         try:
             with warnings.catch_warnings():
                 warnings.simplefilter("error")
@@ -346,7 +489,7 @@ class HighestDensityContour(Contour):
                           "setting n_years to a smaller value.",
                           RuntimeWarning, stacklevel=4)
 
-        # Calculate fm from probability per cell
+        # Calculate fm from probability per cell.
         fm = prob_m
         for delta in deltas:
             fm /= delta
@@ -357,10 +500,10 @@ class HighestDensityContour(Contour):
         labeled_array, n_modes = ndi.label(HDC, structure=structure)
 
         coordinates = []
-        # Iterate over all partial contours, start at 1
+        # Iterate over all partial contours and start at 1.
         for i in range(1, n_modes+1):
             # Array of arrays with same length, one per dimension
-            # containing the indice of the contour
+            # containing the indice of the contour.
             partial_contour_indice = np.nonzero(labeled_array == i)
 
             # Calculate the values corresponding to the indice
@@ -397,9 +540,9 @@ class HighestDensityContour(Contour):
 
         Parameters
         ----------
-        array : ndarray,
+        array : ndarray
             Array of arbitrary shape with all values >= 0.
-        limit : float,
+        limit : float
             limit to sum up to.
 
         Returns
@@ -434,9 +577,9 @@ class HighestDensityContour(Contour):
 
         summed_fields = np.zeros(array.shape)
 
-        summed_fields[np.unravel_index(summed_flat_inds, dims=array.shape)] = 1
+        summed_fields[np.unravel_index(summed_flat_inds, shape=array.shape)] = 1
 
-        last_summed = array[np.unravel_index(summed_flat_inds[-1], dims=array.shape)]
+        last_summed = array[np.unravel_index(summed_flat_inds[-1], shape=array.shape)]
 
 
         return summed_fields, last_summed
